@@ -2,6 +2,14 @@
 
 // Exercised in md5 test suite
 
+Pl_Discard::Members::Members()
+{
+}
+
+Pl_Discard::Members::~Members()
+{
+}
+
 Pl_Discard::Pl_Discard() :
     Pipeline("discard", 0)
 {
@@ -12,7 +20,7 @@ Pl_Discard::~Pl_Discard()
 }
 
 void
-Pl_Discard::write(unsigned char* buf, int len)
+Pl_Discard::write(unsigned char* buf, size_t len)
 {
 }
 
