@@ -1,4 +1,5 @@
 #include <qpdf/QPDFObjGen.hh>
+#include <qpdf/QUtil.hh>
 
 QPDFObjGen::QPDFObjGen() :
     obj(0),
@@ -42,3 +43,10 @@ std::ostream& operator<<(std::ostream& os, const QPDFObjGen& og)
     os << og.obj << "," << og.gen;
     return os;
 }
+
+std::string
+QPDFObjGen::unparse() const
+{
+    return QUtil::int_to_string(this->obj) + "," +
+        QUtil::int_to_string(this->gen);
+}
