@@ -693,22 +693,24 @@ static const u32 rcon[] =
   /* for 128-bit blocks, Rijndael never uses more than 10 rcon values */
 };
 
-#define GETU32(plaintext) (((u32)(plaintext)[0] << 24) ^ \
-                    ((u32)(plaintext)[1] << 16) ^ \
-                    ((u32)(plaintext)[2] <<  8) ^ \
-                    ((u32)(plaintext)[3]))
+#define GETU32(plaintext) \
+              ((static_cast<u32>((plaintext)[0]) << 24) ^ \
+               (static_cast<u32>((plaintext)[1]) << 16) ^ \
+               (static_cast<u32>((plaintext)[2]) <<  8) ^ \
+               (static_cast<u32>((plaintext)[3])))
 
-#define PUTU32(ciphertext, st) { (ciphertext)[0] = (u8)((st) >> 24); \
-                         (ciphertext)[1] = (u8)((st) >> 16); \
-                         (ciphertext)[2] = (u8)((st) >>  8); \
-                         (ciphertext)[3] = (u8)(st); }
+#define PUTU32(ciphertext, st) { \
+                    (ciphertext)[0] = static_cast<u8>((st) >> 24); \
+                    (ciphertext)[1] = static_cast<u8>((st) >> 16); \
+                    (ciphertext)[2] = static_cast<u8>((st) >>  8); \
+                    (ciphertext)[3] = static_cast<u8>(st); }
 
 /**
  * Expand the cipher key into the encryption key schedule.
  *
  * @return the number of rounds for the given cipher key size.
  */
-int rijndaelSetupEncrypt(u32 *rk, const u8 *key, int keybits)
+unsigned int rijndaelSetupEncrypt(u32 *rk, const u8 *key, size_t keybits)
 {
   int i = 0;
   u32 temp;
@@ -797,9 +799,9 @@ int rijndaelSetupEncrypt(u32 *rk, const u8 *key, int keybits)
  *
  * @return the number of rounds for the given cipher key size.
  */
-int rijndaelSetupDecrypt(u32 *rk, const u8 *key, int keybits)
+unsigned int rijndaelSetupDecrypt(u32 *rk, const u8 *key, size_t keybits)
 {
-  int nrounds, i, j;
+  unsigned int nrounds, i, j;
   u32 temp;
 
   /* expand the cipher key: */
@@ -840,7 +842,8 @@ int rijndaelSetupDecrypt(u32 *rk, const u8 *key, int keybits)
   return nrounds;
 }
 
-void rijndaelEncrypt(const u32 *rk, int nrounds, const u8 plaintext[16],
+void rijndaelEncrypt(const u32 *rk, unsigned int nrounds,
+                     const u8 plaintext[16],
   u8 ciphertext[16])
 {
   u32 s0, s1, s2, s3, t0, t1, t2, t3;
@@ -1022,7 +1025,8 @@ void rijndaelEncrypt(const u32 *rk, int nrounds, const u8 plaintext[16],
   PUTU32(ciphertext + 12, s3);
 }
 
-void rijndaelDecrypt(const u32 *rk, int nrounds, const u8 ciphertext[16],
+void rijndaelDecrypt(const u32 *rk, unsigned int nrounds,
+                     const u8 ciphertext[16],
   u8 plaintext[16])
 {
   u32 s0, s1, s2, s3, t0, t1, t2, t3;
